#target illustrator

'Author: Oleg Butrin (wp.me/p0Xz) 2019'
'Contact: https://www.upwork.com/fl/olegbutrin'

var ActionFile = 'action.aiatxt';

String.prototype.trim = function () {
	return this.replace(/^\s|\s$/gim, '');
}

Array.prototype.contains = function (element) {
	for (var i = 0; i < this.length; i++) {
		if (this[i] === element) {
			return true;
		};
	};
	return false;
}

File.prototype.readTXT = function () {
	var result = '';
	this.encoding = 'LATIN1';
	this.open('r');
	result = this.read();
	this.close();
	return result;
}

File.prototype.writeTXT = function (str) {
	this.encoding = 'LATIN1';
	this.open('w');
	this.write(str);
	this.close();
}

function getScriptPath () {
	return File($.fileName).parent;
}

function toHex(s) {
	// utf8 to latin1
	var s = unescape(encodeURIComponent(s))
	var h = ''
	for (var i = 0; i < s.length; i++) {
		h += s.charCodeAt(i).toString(16)
	}
	return h
}

function getLayerByName (obj, name) {
	if (obj.hasOwnProperty('layers')) {
		for (var i = 0; i < obj.layers.length; i++) {
			if (obj.layers[i].name == name) {
				return obj.layers[i];
			}
		}
	}
	return null;
}

function getArtboardIndex (doc, name) {
	for (var i = 0; i < doc.artboards.length; i++) {
		if (doc.artboards[i].name == name) {
			return i;
		}
	}
	return null;
}

function getExportPresetIndex (name) {
	for (var i = 0; i < app.PDFPresetsList.length; i++) {
		if (app.PDFPresetsList[i] == name) {
			return i;
		}
	}
	return null;
}

function callAction (pdfFile, artindex, preset) {
	var setname = 'OLEGBUTRIN';
	var actname = 'SAVETOPDF';
	var epindex = getExportPresetIndex(preset) + 2;
	var sfile = File(getScriptPath() + '/' + ActionFile);
	var dfile = File(getScriptPath() + '/act.aia');
	
	var acttext = sfile.readTXT();
	acttext = acttext.replace('{{setname_l}}', setname.length.toString());
	acttext = acttext.replace('{{setname}}', toHex(setname));
	acttext = acttext.replace('{{actname_l}}', actname.length.toString());
	acttext = acttext.replace('{{actname}}', toHex(actname));
	acttext = acttext.replace('{{optindex}}', epindex.toString());
	acttext = acttext.replace('{{pdfname_l}}', pdfFile.fsName.length.toString());
	acttext = acttext.replace('{{pdfname}}', toHex(pdfFile.fsName));
	acttext = acttext.replace('{{artindex_l}}', artindex.toString().length.toString());
	acttext = acttext.replace('{{artindex}}', toHex(artindex.toString()));
	dfile.writeTXT(acttext);

	app.loadAction(dfile);
	app.doScript(actname, setname);
	app.unloadAction(setname, '');

	dfile.remove();
}

function exportArtboard (doc, artboard, preset, folder) {
	var artindex = getArtboardIndex(doc, artboard.name);
	var pdfName = doc.name.replace(/\.[^\.]+$/gim, '') + '-' + artboard.name;
	var pdfFile = File(folder + '/' + pdfName + '.pdf');
	callAction(pdfFile, artindex + 1, preset);
}

function UI () {
	this.status = false;
	this.preset = null;
	this.folder = null;
	this.window = new Window ('dialog');

	var po = this;

	var grp_main = po.window.add('group {orientation: "column", alignment: "fill", alignChildren: ["fill", "fill"]}');
	var grp_folder = grp_main.add('group {orientation: "row", alignment: "fill", alignChildren: ["fill", "fill"]}');
	var grp_preset = grp_main.add('group {orientation: "row", alignment: "fill", alignChildren: ["fill", "fill"]}');
	var grp_buttons = grp_main.add('group {orientation: "row", alignment: "fill", alignChildren: ["center", "center"]}');

	var btn_folder = grp_folder.add('button');
	var txt_folder = grp_folder.add('edittext', undefined, '', {readonly: true});

	var lbl_preset = grp_preset.add('statictext');
	var ddl_preset = grp_preset.add('dropdownlist');

	var btn_export = grp_buttons.add('button');
	var btn_cancel = grp_buttons.add('button');

	grp_main.margins = 12;

	btn_folder.minimumSize.width = 128;
	btn_folder.maximumSize.width = 128;
	lbl_preset.minimumSize.width = 128;
	lbl_preset.maximumSize.width = 128;
	txt_folder.minimumSize.width = 240;
	txt_folder.maximumSize.width = 240;
	ddl_preset.minimumSize.width = 240;
	ddl_preset.maximumSize.width = 240;

	lbl_preset.justify = 'right';

	po.window.text = 'Export artboards to pdf';
	txt_folder.text = 'Press button and select export folder';
	btn_folder.text = 'Select Folder';
	lbl_preset.text = 'PDF Export Preset:';
	btn_export.text = 'Export';
	btn_cancel.text = 'Cancel';
	po.window.cancelElement = btn_cancel;

	this.window.onShow = function () {
		this.window.layout.layout();
	}

	btn_folder.onClick = function () {
		var fldr = getScriptPath().selectDlg('Select folder for export artboards as PDF:');
		if (!fldr) {
			return false;
		}
		po.folder = fldr;
		txt_folder.text = decodeURI(po.folder.name);
		txt_folder.helpTip = decodeURI(po.folder.fsName);
	}

	ddl_preset.onChange = function () {
		po.preset = ddl_preset.selection.text;
	}

	btn_export.onClick = function () {
		if (po.folder == null || po.preset == null) {
			return false;
		}
		po.status = true;
		po.window.close();
	}

	this.show = function () {
		ddl_preset.removeAll();
		for (var i = 0; i < app.PDFPresetsList.length; i++) {
			ddl_preset.add('item', app.PDFPresetsList[i]);
		}
		ddl_preset.selection = ddl_preset.items[0];
		ddl_preset.notify();
		this.window.show();
	}
}

function main () {
	if (app.documents.length == 0) {
		alert('No documents are open!');
		return false;
	}
	var doc = app.activeDocument;
	if (!doc.hasOwnProperty('path') || !Folder(doc.path).exists) {
		alert('Please save document before using script!');
		return false;
	}
	
	var ui = new UI();
	ui.show();

	if (ui.status && ui.folder && ui.preset) {
		for (var i = 0; i < doc.artboards.length; i++) {
			exportArtboard(doc, doc.artboards[i], ui.preset, ui.folder);
		}
	} else {
		return false;
	}
	
	return true;
}

main();